#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdint.h>
#include "MPL115A2.h"

/** Default constructor, uses default I2C address.
 * @see MPL115A2_DEFAULT_ADDRESS
 */
MPL115A2::MPL115A2() {
    devAddr = MPL115A2_DEFAULT_ADDRESS;
}

/** Specific address constructor.
 * @param address I2C address
 * @see MPL115A2_DEFAULT_ADDRESS
 */
MPL115A2::MPL115A2(uint8_t address) {
    devAddr = address;
}

void MPL115A2::readCoefficients() {
	uint8_t data[8];
	int16_t a0coeff;
	int16_t b1coeff;
	int16_t b2coeff;
	int16_t c12coeff;
	
	I2C_RW::readByte(devAddr, MPL115A2_a0_MSB, 8, &data[0]);
	
	a0coeff = (data[0] << 8 | data[1]);
	a0 = (float)a0coeff / 8;
	
	b1coeff = (data[2] << 8 | data[3]);
	b1 = (float)b1coeff / 8192;
	
	b2coeff = (data[4] << 8 | data[5]);
	b2 = (float)b2coeff / 16384;
	
	c12coeff = (data[6] << 8 | data[7]) >> 2;
	c12 = (float)c12coeff / 4194304.0;
}

/** Just to see if I did it right
 *  Printf - did it work
 */
void MPL115A2::setup() {
	MPL115A2::readCoefficients();
	//printf("A0 = %f\n", a0);
	//printf("B1 = %f\n", b1);
	//printf("B2 = %f\n", b2);
	//printf("C12 = %f\n", c12);
}

/**************************************************************************/
/*!
    @brief  Gets the floating-point pressure level in kPa
*/
/**************************************************************************/
float MPL115A2::getPressure() {
	float     pressureComp,centigrade;

	getPT(&pressureComp, &centigrade);
	return pressureComp;
}


/**************************************************************************/
/*!
    @brief  Gets the floating-point temperature in Centigrade
*/
/**************************************************************************/
float MPL115A2::getTemperature() {
	float     pressureComp, centigrade;

	getPT(&pressureComp, &centigrade);
	return centigrade;
}

/**************************************************************************/
/*!
    @brief  Gets both at once and saves a little time
*/
/**************************************************************************/
void MPL115A2::getPT(float *P, float *T) {
	uint8_t dataRead[4], dataWrite = 0;
	uint16_t 	pressure, temp = 0;
	float     pressureComp;

	// Get raw pressure and temperature settings
	I2C_RW::writeByte(devAddr, MPL115A2_CONVERT, &dataWrite);

	// Wait a bit for the conversion to complete (3ms max)
	sleep(1);

	I2C_RW::readByte(devAddr, MPL115A2_Padc_MSB , 4, &dataRead[0]);
	
	pressure = (dataRead[0] << 8 | dataRead[1]) >> 6;
	temp = (dataRead[2] << 8 | dataRead[3]) >> 6;
	
	// See datasheet p.6 for evaluation sequence
	pressureComp = a0 + (b1 + c12 * temp ) * pressure + b2 * temp;

	// Return pressure and temperature as floating point values
	*P = ((65.0 / 1023.0) * pressureComp) + 50.0;        // kPa
	*T = ((float) temp - 498.0) / -5.35 +25.0;           // C

}
